==> knt_monetize_bloc-2.4.5.tar.gz: lib/src/bloc/revcat_subscription_bloc.dart <==
import 'package:flutter/services.dart';
import 'package:flutter_inapp_purchase/flutter_inapp_purchase.dart';
import 'package:intl/intl.dart';
import 'package:knt_bloc/knt_bloc.dart';
import 'package:purchases_flutter/purchases_flutter.dart';

import '../model/subscription_item.dart';
import '../repository/monetized_repo.dart';
import '../service/revcat_iap_service.dart';

class RevCatSubscriptionBloc extends BaseBloc {
  final RevCatIapService _iapService;
  final MonetizedRepo _monetizedRepo;

  // In-app-purchase service
  late bool _purchased;

  RevCatSubscriptionBloc(this._iapService, this._monetizedRepo) {
    watch<RCConnectSubscriptionServiceEvent>(
      _handleRCConnectSubscriptionServiceEvent,
      transformer: droppable(),
    );
    watch<RCCheckEntitlementEvent>(
      _handleRCCheckEntitlementEvent,
      transformer: restartable(),
    );
    watch<RCRequestSubscriptionEvent>(
      _handleRCRequestSubscriptionEvent,
      transformer: sequential(),
    );
    watch<RCRestoreSubscriptionEvent>(
      _handleRCRestoreSubscriptionEvent,
      transformer: restartable(),
    );
    watch<RCFetchOfferingsEvent>(
      _handleRCFetchOfferingsEvent,
      transformer: restartable(),
    );
    watch<_RCUpdatePurchaseInfoEvent>(
      _handleRCUpdatePurchaseInfoEvent,
      transformer: sequential(),
    );
    add(RCConnectSubscriptionServiceEvent());
  }

  @override
  void reset() {
    _purchased = false;
  }

  @override
  Future<void> handleBaseErrorEvent(
    BaseErrorEvent event,
    Emitter<BaseState> emit,
  ) async {
    if (event.tag == (RCFetchOfferingsEvent).toString()) {
      emit(RCFetchedOfferingsFailureState());
      return;
    }
    if (event.tag == (RCRestoreSubscriptionEvent).toString()) {
      emit(RCFetchedRestoreFailureState());
      return;
    }
    if (event.tag == (RCRequestSubscriptionEvent).toString()) {
      emit(RCUnPurchasedState());
      return;
    }
    return super.handleBaseErrorEvent(event, emit);
  }

  Future<void> _handleRCConnectSubscriptionServiceEvent(
    RCConnectSubscriptionServiceEvent event,
    Emitter<BaseState> emit,
  ) async {
    // Get subscriber info from RevenueCat
    final purchaserInfo = await Purchases.getPurchaserInfo();
    final subscriberFirstSeenDate = purchaserInfo.firstSeen.parse();
    final requestDate = purchaserInfo.requestDate.parse();
    // User is considered as a new RevenueCat user if the first seen date is
    // not longer than 120 seconds compare to the request date
    final userNewToRevenueCat =
        requestDate.isAfterInRange(120, subscriberFirstSeenDate);
    final hasEntitlement =
        purchaserInfo.entitlements.all[_iapService.entitlementID]?.isActive ??
            false;
    if (hasEntitlement) {
      _purchased = true;
      add(_RCUpdatePurchaseInfoEvent());
      return;
    }
    if (!event.needSyncFromOldService || !userNewToRevenueCat) {
      _purchased = false;
      add(_RCUpdatePurchaseInfoEvent());
      return;
    }

    // Request old system to check available SKUs, then sync to RevenueCat if
    // user has subscription
    FlutterInappPurchase.instance.initialize().then((value) async {
      await cancelableFuture(
          FlutterInappPurchase.instance.getAvailablePurchases());
      await FlutterInappPurchase.instance.finalize();
    }).onError((error, stackTrace) {
      _purchased = false;
      add(_RCUpdatePurchaseInfoEvent());
    });
  }

  Future<void> _handleRCCheckEntitlementEvent(
    RCCheckEntitlementEvent event,
    Emitter<BaseState> emit,
  ) async {
    final purchaserInfo = await Purchases.getPurchaserInfo();
    _updatePurchaserInfo(purchaserInfo);
  }

  Future<void> _handleRCRequestSubscriptionEvent(
    RCRequestSubscriptionEvent event,
    Emitter<BaseState> emit,
  ) async {
    emit(RCPurchasingState());
    final offerings = await Purchases.getOfferings();
    for (final package in offerings.current?.availablePackages ?? <Package>[]) {
      if (package.product.identifier == event.subItem.sku) {
        try {
          final purchaserInfo = await Purchases.purchasePackage(package);
          _updatePurchaserInfo(purchaserInfo);
        } on PlatformException catch (e) {
          _purchased = PurchasesErrorHelper.getErrorCode(e) ==
              PurchasesErrorCode.productAlreadyPurchasedError;
          add(_RCUpdatePurchaseInfoEvent());
        } on Exception {
          // Ignored error caused
          _purchased = false;
          add(_RCUpdatePurchaseInfoEvent());
        }
        return;
      }
    }
  }

  Future<void> _handleRCRestoreSubscriptionEvent(
    RCRestoreSubscriptionEvent event,
    Emitter<BaseState> emit,
  ) async {
    emit(RCRestoringState());
    final purchaserInfo = await Purchases.restoreTransactions();
    _updatePurchaserInfo(purchaserInfo);
  }

  Future<void> _handleRCFetchOfferingsEvent(
    RCFetchOfferingsEvent event,
    Emitter<BaseState> emit,
  ) async {
    emit(RCFetchingOfferingsState());
    final offerings = await Purchases.getOfferings();
    final subList = _iapService
        .packageConverter(offerings.current?.availablePackages ?? []);
    if (subList.isEmpty) {
      throw KntException(KntIssue.undefined);
    }
    emit(RCFetchedOfferingsState(subList));
  }

  Future<void> _handleRCUpdatePurchaseInfoEvent(
    _RCUpdatePurchaseInfoEvent event,
    Emitter<BaseState> emit,
  ) async {
    await _monetizedRepo.savePurchasedIapStatus(purchased: _purchased);
    if (_purchased) {
      emit(RCPurchasedState());
    } else {
      emit(RCUnPurchasedState());
    }
  }

  void _updatePurchaserInfo(PurchaserInfo purchaserInfo) {
    _purchased =
        purchaserInfo.entitlements.all[_iapService.entitlementID]?.isActive ??
            false;
    add(_RCUpdatePurchaseInfoEvent());
  }
}

// region EVENT
class RCConnectSubscriptionServiceEvent extends BaseEvent {
  final bool needSyncFromOldService;

  const RCConnectSubscriptionServiceEvent({this.needSyncFromOldService = false});
}

class RCCheckEntitlementEvent extends BaseEvent {}

class RCRequestSubscriptionEvent extends BaseEvent {
  final SubscriptionItem subItem;

  const RCRequestSubscriptionEvent(this.subItem);

  @override
  String toString() =>
      'RCRequestSubscriptionEvent: item has sku ${subItem.sku}';
}

class RCRestoreSubscriptionEvent extends BaseEvent {}

class RCFetchOfferingsEvent extends BaseEvent {}

class _RCUpdatePurchaseInfoEvent extends BaseEvent {}
// endregion

// region STATE
class RCPurchasingState extends BaseState {}

class RCRestoringState extends BaseState {}

class RCPurchasedState extends BaseState {}

class RCUnPurchasedState extends BaseState {}

class RCFetchingOfferingsState extends BaseState {}

class RCFetchedOfferingsState
    extends BaseHasResultState<List<SubscriptionItem>> {
  const RCFetchedOfferingsState(List<SubscriptionItem> subList)
      : super(subList);
}

class RCFetchedOfferingsFailureState extends BaseErrorState {
  RCFetchedOfferingsFailureState()
      : super(exception: KntException(KntIssue.undefined));
}

class RCFetchedRestoreFailureState extends BaseState {}
// endregion

extension DateUtils on DateTime {
  bool isAfterInRange(int acceptableRangeInSeconds, DateTime? date) {
    if (date == null || this.isBefore(date)) {
      return false;
    }
    return this.difference(date).inSeconds <= acceptableRangeInSeconds;
  }
}

extension InversionDate on String {
  static const revenueCatDateTimePattern = 'yyyy-MM-dd\'T\'HH:mm:ss\'Z\'';
  static const backupRevenueCatDateTimePattern =
      'yyyy-MM-dd\'T\'HH:mm:ss.SSS\'Z\'';

  DateTime parse() {
    try {
      return DateFormat(revenueCatDateTimePattern).parse(this);
    } on Exception {
      return DateFormat(backupRevenueCatDateTimePattern).parse(this);
    }
  }
}

==> knt_monetize_bloc-3.0.0.tar.gz: lib/src/bloc/revcat_subscription_bloc.dart <==
import 'package:flutter/services.dart';
import 'package:flutter_inapp_purchase/flutter_inapp_purchase.dart';
import 'package:intl/intl.dart';
import 'package:knt_bloc/knt_bloc.dart';
import 'package:purchases_flutter/purchases_flutter.dart';

import '../model/subscription_item.dart';
import '../repository/monetized_repo.dart';
import '../service/revcat_iap_service.dart';

class RevCatSubscriptionBloc extends BaseBloc {
  final RevCatIapService _iapService;
  final MonetizedRepo _monetizedRepo;

  // In-app-purchase service
  late bool _purchased;

  RevCatSubscriptionBloc(this._iapService, this._monetizedRepo) {
    watch<RCConnectSubscriptionServiceEvent>(
      _handleRCConnectSubscriptionServiceEvent,
      transformer: droppable(),
    );
    watch<RCCheckEntitlementEvent>(
      _handleRCCheckEntitlementEvent,
      transformer: restartable(),
    );
    watch<RCRequestSubscriptionEvent>(
      _handleRCRequestSubscriptionEvent,
      transformer: sequential(),
    );
    watch<RCRestoreSubscriptionEvent>(
      _handleRCRestoreSubscriptionEvent,
      transformer: restartable(),
    );
    watch<RCFetchOfferingsEvent>(
      _handleRCFetchOfferingsEvent,
      transformer: restartable(),
    );
    watch<_RCUpdatePurchaseInfoEvent>(
      _handleRCUpdatePurchaseInfoEvent,
      transformer: sequential(),
    );
    add(RCConnectSubscriptionServiceEvent());
  }

  @override
  void reset() {
    _purchased = false;
  }

  @override
  Future<void> handleBaseErrorEvent(
    BaseErrorEvent event,
    Emitter<BaseState> emit,
  ) async {
    if (event.tag == (RCFetchOfferingsEvent).toString()) {
      emit(RCFetchedOfferingsFailureState());
      return;
    }
    if (event.tag == (RCRestoreSubscriptionEvent).toString()) {
      emit(RCFetchedRestoreFailureState());
      return;
    }
    if (event.tag == (RCRequestSubscriptionEvent).toString()) {
      emit(RCUnPurchasedState());
      return;
    }
    return super.handleBaseErrorEvent(event, emit);
  }

  Future<void> _handleRCConnectSubscriptionServiceEvent(
    RCConnectSubscriptionServiceEvent event,
    Emitter<BaseState> emit,
  ) async {
    // Get subscriber info from RevenueCat
    final customerInfo = await Purchases.getCustomerInfo();
    final subscriberFirstSeenDate = customerInfo.firstSeen.parse();
    final requestDate = customerInfo.requestDate.parse();
    // User is considered as a new RevenueCat user if the first seen date is
    // not longer than 120 seconds compare to the request date
    final userNewToRevenueCat =
        requestDate.isAfterInRange(120, subscriberFirstSeenDate);
    final hasEntitlement =
        customerInfo.entitlements.all[_iapService.entitlementID]?.isActive ??
            false;
    if (hasEntitlement) {
      _purchased = true;
      add(_RCUpdatePurchaseInfoEvent());
      return;
    }
    if (!event.needSyncFromOldService || !userNewToRevenueCat) {
      _purchased = false;
      add(_RCUpdatePurchaseInfoEvent());
      return;
    }

    // Request old system to check available SKUs, then sync to RevenueCat if
    // user has subscription
    FlutterInappPurchase.instance.initialize().then((value) async {
      await cancelableFuture(
          FlutterInappPurchase.instance.getAvailablePurchases());
      await FlutterInappPurchase.instance.finalize();
    }).onError((error, stackTrace) {
      _purchased = false;
      add(_RCUpdatePurchaseInfoEvent());
    });
  }

  Future<void> _handleRCCheckEntitlementEvent(
    RCCheckEntitlementEvent event,
    Emitter<BaseState> emit,
  ) async {
    final customerInfo = await Purchases.getCustomerInfo();
    _updateCustomerInfo(customerInfo);
  }

  Future<void> _handleRCRequestSubscriptionEvent(
    RCRequestSubscriptionEvent event,
    Emitter<BaseState> emit,
  ) async {
    emit(RCPurchasingState());
    final offerings = await Purchases.getOfferings();
    for (final package in offerings.current?.availablePackages ?? <Package>[]) {
      if (package.storeProduct.identifier == event.subItem.sku) {
        try {
          final customerInfo = await Purchases.purchasePackage(package);
          _updateCustomerInfo(customerInfo);
        } on PlatformException catch (e) {
          _purchased = PurchasesErrorHelper.getErrorCode(e) ==
              PurchasesErrorCode.productAlreadyPurchasedError;
          add(_RCUpdatePurchaseInfoEvent());
        } on Exception {
          // Ignored error caused
          _purchased = false;
          add(_RCUpdatePurchaseInfoEvent());
        }
        return;
      }
    }
  }

  Future<void> _handleRCRestoreSubscriptionEvent(
    RCRestoreSubscriptionEvent event,
    Emitter<BaseState> emit,
  ) async {
    emit(RCRestoringState());
    final customerInfo = await Purchases.restorePurchases();
    _updateCustomerInfo(customerInfo);
  }

  Future<void> _handleRCFetchOfferingsEvent(
    RCFetchOfferingsEvent event,
    Emitter<BaseState> emit,
  ) async {
    emit(RCFetchingOfferingsState());
    final offerings = await Purchases.getOfferings();
    final subList = _iapService
        .packageConverter(offerings.current?.availablePackages ?? []);
    if (subList.isEmpty) {
      throw KntException(KntIssue.undefined);
    }
    emit(RCFetchedOfferingsState(subList));
  }

  Future<void> _handleRCUpdatePurchaseInfoEvent(
    _RCUpdatePurchaseInfoEvent event,
    Emitter<BaseState> emit,
  ) async {
    await _monetizedRepo.savePurchasedIapStatus(purchased: _purchased);
    if (_purchased) {
      emit(RCPurchasedState());
    } else {
      emit(RCUnPurchasedState());
    }
  }

  void _updateCustomerInfo(CustomerInfo customerInfo) {
    _purchased =
        customerInfo.entitlements.all[_iapService.entitlementID]?.isActive ??
            false;
    add(_RCUpdatePurchaseInfoEvent());
  }
}

// region EVENT
class RCConnectSubscriptionServiceEvent extends BaseEvent {
  final bool needSyncFromOldService;

  const RCConnectSubscriptionServiceEvent(
      {this.needSyncFromOldService = false});
}

class RCCheckEntitlementEvent extends BaseEvent {}

class RCRequestSubscriptionEvent extends BaseEvent {
  final SubscriptionItem subItem;

  const RCRequestSubscriptionEvent(this.subItem);

  @override
  String toString() =>
      'RCRequestSubscriptionEvent: item has sku ${subItem.sku}';
}

class RCRestoreSubscriptionEvent extends BaseEvent {}

class RCFetchOfferingsEvent extends BaseEvent {}

class _RCUpdatePurchaseInfoEvent extends BaseEvent {}
// endregion

// region STATE
class RCPurchasingState extends BaseState {}

class RCRestoringState extends BaseState {}

class RCPurchasedState extends BaseState {}

class RCUnPurchasedState extends BaseState {}

class RCFetchingOfferingsState extends BaseState {}

class RCFetchedOfferingsState
    extends BaseHasResultState<List<SubscriptionItem>> {
  const RCFetchedOfferingsState(List<SubscriptionItem> subList)
      : super(subList);
}

class RCFetchedOfferingsFailureState extends BaseErrorState {
  RCFetchedOfferingsFailureState()
      : super(exception: KntException(KntIssue.undefined));
}

class RCFetchedRestoreFailureState extends BaseState {}
// endregion

extension DateUtils on DateTime {
  bool isAfterInRange(int acceptableRangeInSeconds, DateTime? date) {
    if (date == null || this.isBefore(date)) {
      return false;
    }
    return this.difference(date).inSeconds <= acceptableRangeInSeconds;
  }
}

extension InversionDate on String {
  static const revenueCatDateTimePattern = 'yyyy-MM-dd\'T\'HH:mm:ss\'Z\'';
  static const backupRevenueCatDateTimePattern =
      'yyyy-MM-dd\'T\'HH:mm:ss.SSS\'Z\'';

  DateTime parse() {
    try {
      return DateFormat(revenueCatDateTimePattern).parse(this);
    } on Exception {
      return DateFormat(backupRevenueCatDateTimePattern).parse(this);
    }
  }
}
